@@ -37,6 +37,7 @@ gpointer rb_copy_function (gpointer data);
 int rb_gvalue_compare (GValue *a, GValue *b);
 
 int rb_compare_gtimeval (GTimeVal *a, GTimeVal *b);
+int rb_safe_strcmp (const char *a, const char *b);
 char *rb_make_duration_string (guint duration);
 char *rb_make_elapsed_time_string (guint elapsed, guint duration, gboolean show_remaining);
 
@@ -58,6 +59,8 @@ gboolean rb_string_list_equal (GList *a, GList *b);
 void rb_list_deep_free (GList *list);
 GList* rb_string_list_copy (GList *list);
 
+gboolean rb_str_in_strv (const char *needle, char **haystack);
+
 GList* rb_collate_hash_table_keys (GHashTable *table);
 GList* rb_collate_hash_table_values (GHashTable *table);
 
