@@ -25,6 +25,7 @@ popup_ui = """
   <popup name="MagnatuneSourceViewPopup">
     <menuitem name="AddToQueueLibraryPopup" action="AddToQueue"/>
     <menuitem name="MagnatunePurchaseAlbum" action="MagnatunePurchaseAlbum"/>
+    <menuitem name="MagnatunePurchaseCD" action="MagnatunePurchaseCD"/>
     <menuitem name="MagnatuneArtistInfo" action="MagnatuneArtistInfo"/>
     <menuitem name="MagnatuneCancelDownload" action="MagnatuneCancelDownload"/>
     <separator/>
@@ -60,7 +61,7 @@ class Magnatune(rb.Plugin):
 		rb.Plugin.__init__(self)
 		
 	def activate(self, shell):
-		self.shell = shell # so buy_track can update the progress bar
+		self.shell = shell # so the source can update the progress bar
 		self.db = shell.get_property("db")
 		self.keyring = None
 
@@ -78,9 +79,14 @@ class Magnatune(rb.Plugin):
 		action = gtk.Action('MagnatunePurchaseAlbum', _('Purchase Album'),
 				_("Purchase this album from Magnatune"),
 				'gtk-save')
-		action.connect('activate', lambda a: self.shell.get_property("selected-source").purchase_tracks())
+		action.connect('activate', lambda a: self.shell.get_property("selected-source").purchase_album())
 		self.action_group = gtk.ActionGroup('MagnatunePluginActions')
 		self.action_group.add_action(action)
+		action = gtk.Action('MagnatunePurchaseCD', _('Purchase Physical CD'),
+				_("Purchase a physical CD from Magnatune"),
+				'gtk-cdrom')
+		action.connect('activate', lambda a: self.shell.get_property("selected-source").buy_cd())
+		self.action_group.add_action(action)
 		action = gtk.Action('MagnatuneArtistInfo', _('Artist Information'),
 				_("Get information about this artist"),
 				'gtk-info')
