@@ -29,7 +29,7 @@
 
 #include "rhythmdb-property-model.h"
 #include "rb-debug.h"
-#include "gsequence.h"
+#include "eggsequence.h"
 #include "rb-refstring.h"
 #include "rb-tree-dnd.h"
 
@@ -48,6 +48,7 @@ typedef struct {
 	guint refcount;
 } RhythmDBPropertyModelEntry;
 
+static void rhythmdb_property_model_dispose (GObject *object);
 static void rhythmdb_property_model_finalize (GObject *object);
 static void rhythmdb_property_model_set_property (GObject *object,
 					       guint prop_id,
@@ -155,7 +156,7 @@ struct RhythmDBPropertyModelPrivate
 
 	guint stamp;
 
-	GSequence *properties;
+	EggSequence *properties;
 	GHashTable *reverse_map;
 
 	RhythmDBPropertyModelEntry *all;
@@ -206,6 +207,7 @@ rhythmdb_property_model_class_init (RhythmDBPropertyModelClass *klass)
 	object_class->set_property = rhythmdb_property_model_set_property;
 	object_class->get_property = rhythmdb_property_model_get_property;
 
+	object_class->dispose = rhythmdb_property_model_dispose;
 	object_class->finalize = rhythmdb_property_model_finalize;
 
 	rhythmdb_property_model_signals[PRE_ROW_DELETION] =
@@ -420,7 +422,7 @@ rhythmdb_property_model_init (RhythmDBPropertyModel *model)
 
 	model->priv->stamp = g_random_int ();
 
-	model->priv->properties = g_sequence_new (NULL);
+	model->priv->properties = egg_sequence_new (NULL);
 	model->priv->reverse_map = g_hash_table_new (g_str_hash, g_str_equal);
 	model->priv->entries = g_hash_table_new (g_direct_hash, g_direct_equal);
 
@@ -428,12 +430,45 @@ rhythmdb_property_model_init (RhythmDBPropertyModel *model)
 	model->priv->all->string = rb_refstring_new (_("All"));
 }
 
+static void
+_prop_model_entry_cleanup (RhythmDBPropertyModelEntry *prop, gpointer data)
+{
+	rb_refstring_unref (prop->string);
+	rb_refstring_unref (prop->sort_string);
+	g_free (prop);
+}
+
+static void
+rhythmdb_property_model_dispose (GObject *object)
+{
+	RhythmDBPropertyModel *model;
+
+	g_return_if_fail (object != NULL);
+	g_return_if_fail (RHYTHMDB_IS_PROPERTY_MODEL (object));
+
+	model = RHYTHMDB_PROPERTY_MODEL (object);
+
+	rb_debug ("disposing property model %p", model);
+
+	g_return_if_fail (model->priv != NULL);
+
+	if (model->priv->syncing_id != 0) {
+		g_source_remove (model->priv->syncing_id);
+		model->priv->syncing_id = 0;
+	}
+
+	if (model->priv->query_model != NULL) {
+		g_object_unref (model->priv->query_model);
+		model->priv->query_model = NULL;
+	}
+
+	G_OBJECT_CLASS (rhythmdb_property_model_parent_class)->dispose (object);
+}
+
 static void
 rhythmdb_property_model_finalize (GObject *object)
 {
 	RhythmDBPropertyModel *model;
-	GSequencePtr ptr;
-	GSequencePtr end_ptr;
 
 	g_return_if_fail (object != NULL);
 	g_return_if_fail (RHYTHMDB_IS_PROPERTY_MODEL (object));
@@ -444,26 +479,14 @@ rhythmdb_property_model_finalize (GObject *object)
 
 	g_return_if_fail (model->priv != NULL);
 
-	if (model->priv->syncing_id != 0)
-		g_source_remove (model->priv->syncing_id);
-
 	g_hash_table_destroy (model->priv->reverse_map);
 
-	end_ptr = g_sequence_get_end_ptr (model->priv->properties);
-	for (ptr = g_sequence_get_begin_ptr (model->priv->properties); ptr != end_ptr;
-	     ptr = g_sequence_ptr_next (ptr)) {
-		RhythmDBPropertyModelEntry *prop = g_sequence_ptr_get_data (ptr);
-		rb_refstring_unref (prop->string);
-		rb_refstring_unref (prop->sort_string);
-		g_free (prop);
-	}
-	g_sequence_free (model->priv->properties);
+	egg_sequence_foreach (model->priv->properties, (GFunc)_prop_model_entry_cleanup, NULL);
+	egg_sequence_free (model->priv->properties);
 
 	g_hash_table_destroy (model->priv->entries);
 
-	if (model->priv->query_model != NULL) {
-		g_object_unref (model->priv->query_model);
-	}
+	g_free (model->priv->all);
 
 	G_OBJECT_CLASS (rhythmdb_property_model_parent_class)->finalize (object);
 }
@@ -563,7 +586,7 @@ rhythmdb_property_model_insert (RhythmDBPropertyModel *model,
 	RhythmDBPropertyModelEntry *prop;
 	GtkTreeIter iter;
 	GtkTreePath *path;
-	GSequencePtr ptr;
+	EggSequenceIter *ptr;
 	const char *propstr;
 
 	propstr = rhythmdb_entry_get_string (entry, model->priv->propid);
@@ -571,7 +594,7 @@ rhythmdb_property_model_insert (RhythmDBPropertyModel *model,
 	model->priv->all->refcount++;
 
 	if ((ptr = g_hash_table_lookup (model->priv->reverse_map, propstr))) {
-		prop = g_sequence_ptr_get_data (ptr);
+		prop = egg_sequence_get (ptr);
 		prop->refcount++;
 		rb_debug ("adding \"%s\": refcount %d", propstr, prop->refcount);
 		return prop;
@@ -584,7 +607,7 @@ rhythmdb_property_model_insert (RhythmDBPropertyModel *model,
 	prop->refcount = 1;
 
 	iter.stamp = model->priv->stamp;
-	ptr = g_sequence_insert_sorted (model->priv->properties, prop,
+	ptr = egg_sequence_insert_sorted (model->priv->properties, prop,
 					(GCompareDataFunc) rhythmdb_property_model_compare,
 					model);
 	g_hash_table_insert (model->priv->reverse_map,
@@ -616,7 +639,7 @@ static void
 rhythmdb_property_model_delete_prop (RhythmDBPropertyModel *model,
 				     const char *propstr)
 {
-	GSequencePtr ptr;
+	EggSequenceIter *ptr;
 	RhythmDBPropertyModelEntry *prop;
 	GtkTreePath *path;
 	GtkTreeIter iter;
@@ -625,7 +648,7 @@ rhythmdb_property_model_delete_prop (RhythmDBPropertyModel *model,
 
 	model->priv->all->refcount--;
 
-	prop = g_sequence_ptr_get_data (ptr);
+	prop = egg_sequence_get (ptr);
 	rb_debug ("deleting \"%s\": refcount: %d", propstr, prop->refcount);
 	prop->refcount--;
 	if (prop->refcount > 0)
@@ -638,7 +661,7 @@ rhythmdb_property_model_delete_prop (RhythmDBPropertyModel *model,
 	g_signal_emit (G_OBJECT (model), rhythmdb_property_model_signals[PRE_ROW_DELETION], 0);
 	gtk_tree_model_row_deleted (GTK_TREE_MODEL (model), path);
 	gtk_tree_path_free (path);
-	g_sequence_remove (ptr);
+	egg_sequence_remove (ptr);
 	g_hash_table_remove (model->priv->reverse_map, propstr);
 	rb_refstring_unref (prop->string);
 	rb_refstring_unref (prop->sort_string);
@@ -651,7 +674,7 @@ rhythmdb_property_model_iter_from_string (RhythmDBPropertyModel *model,
 					  const char *name,
 					  GtkTreeIter *iter)
 {
-	GSequencePtr ptr;
+	EggSequenceIter *ptr;
 
 	if (name == NULL) {
 		if (iter) {
@@ -709,7 +732,7 @@ rhythmdb_property_model_get_iter (GtkTreeModel *tree_model,
 {
 	RhythmDBPropertyModel *model = RHYTHMDB_PROPERTY_MODEL (tree_model);
 	guint index;
-	GSequencePtr ptr;
+	EggSequenceIter *ptr;
 
 	index = gtk_tree_path_get_indices (path)[0];
 
@@ -720,10 +743,10 @@ rhythmdb_property_model_get_iter (GtkTreeModel *tree_model,
 	}
 
 	index--;
-	if (index >= g_sequence_get_length (model->priv->properties))
+	if (index >= egg_sequence_get_length (model->priv->properties))
 		return FALSE;
 
-	ptr = g_sequence_get_ptr_at_pos (model->priv->properties, index);
+	ptr = egg_sequence_get_iter_at_pos (model->priv->properties, index);
 
 	iter->stamp = model->priv->stamp;
 	iter->user_data = ptr;
@@ -744,14 +767,14 @@ rhythmdb_property_model_get_path (GtkTreeModel *tree_model,
 		return gtk_tree_path_new_first ();
 	}
 
-	if (g_sequence_ptr_is_end (iter->user_data))
+	if (egg_sequence_iter_is_end (iter->user_data))
 		return NULL;
 
 	path = gtk_tree_path_new ();
 	if (iter->user_data == model->priv->all)
 		gtk_tree_path_append_index (path, 0);
 	else
-		gtk_tree_path_append_index (path, g_sequence_ptr_get_position (iter->user_data) + 1);
+		gtk_tree_path_append_index (path, egg_sequence_iter_get_position (iter->user_data) + 1);
 	return path;
 }
 
@@ -785,9 +808,9 @@ rhythmdb_property_model_get_value (GtkTreeModel *tree_model,
 	} else {
 		RhythmDBPropertyModelEntry *prop;
 
-		g_return_if_fail (!g_sequence_ptr_is_end (iter->user_data));
+		g_return_if_fail (!egg_sequence_iter_is_end (iter->user_data));
 
-		prop = g_sequence_ptr_get_data (iter->user_data);
+		prop = egg_sequence_get (iter->user_data);
 		switch (column) {
 		case RHYTHMDB_PROPERTY_MODEL_COLUMN_TITLE:
 			g_value_init (value, G_TYPE_STRING);
@@ -816,13 +839,13 @@ rhythmdb_property_model_iter_next (GtkTreeModel  *tree_model,
 	g_return_val_if_fail (iter->stamp == model->priv->stamp, FALSE);
 
 	if (iter->user_data == model->priv->all) {
-		iter->user_data = g_sequence_get_begin_ptr (model->priv->properties);
+		iter->user_data = egg_sequence_get_begin_iter (model->priv->properties);
 	} else {
-		g_return_val_if_fail (!g_sequence_ptr_is_end (iter->user_data), FALSE);
-		iter->user_data = g_sequence_ptr_next (iter->user_data);
+		g_return_val_if_fail (!egg_sequence_iter_is_end (iter->user_data), FALSE);
+		iter->user_data = egg_sequence_iter_next (iter->user_data);
 	}
 
-	return !g_sequence_ptr_is_end (iter->user_data);
+	return !egg_sequence_iter_is_end (iter->user_data);
 }
 
 static gboolean
@@ -858,7 +881,7 @@ rhythmdb_property_model_iter_n_children (GtkTreeModel *tree_model,
 		g_return_val_if_fail (model->priv->stamp == iter->stamp, -1);
 
 	if (iter == NULL)
-		return 1 + g_sequence_get_length (model->priv->properties);
+		return 1 + egg_sequence_get_length (model->priv->properties);
 
 	return 0;
 }
@@ -870,15 +893,15 @@ rhythmdb_property_model_iter_nth_child (GtkTreeModel *tree_model,
 					gint n)
 {
 	RhythmDBPropertyModel *model = RHYTHMDB_PROPERTY_MODEL (tree_model);
-	GSequencePtr child;
+	EggSequenceIter *child;
 
 	if (parent)
 		return FALSE;
 
 	if (n != 0) {
-		child = g_sequence_get_ptr_at_pos (model->priv->properties, n);
+		child = egg_sequence_get_iter_at_pos (model->priv->properties, n);
 
-		if (g_sequence_ptr_is_end (child))
+		if (egg_sequence_iter_is_end (child))
 			return FALSE;
 		iter->user_data = child;
 	} else {
