@@ -17,11 +17,11 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
  *
  */
 
-#include <config.h>
+#include "config.h"
 
 #include <unistd.h>
 #include <string.h>
@@ -32,7 +32,6 @@
 #include <libgnomevfs/gnome-vfs-uri.h>
 #include <totem-pl-parser.h>
 
-#include "rb-stock-icons.h"
 #include "rb-entry-view.h"
 #include "rb-search-entry.h"
 #include "rb-file-helpers.h"
@@ -40,7 +39,6 @@
 #include "rb-dialog.h"
 #include "rb-util.h"
 #include "rb-playlist-source.h"
-#include "rb-playlist-source-recorder.h"
 #include "rb-debug.h"
 #include "eel-gconf-extensions.h"
 #include "rb-song-info.h"
@@ -51,27 +49,29 @@
 
 static void rb_playlist_source_class_init (RBPlaylistSourceClass *klass);
 static void rb_playlist_source_init (RBPlaylistSource *source);
-static GObject *rb_playlist_source_constructor (GType type, guint n_construct_properties,
+static GObject *rb_playlist_source_constructor (GType type,
+						guint n_construct_properties,
 						GObjectConstructParam *construct_properties);
 static void rb_playlist_source_dispose (GObject *object);
 static void rb_playlist_source_finalize (GObject *object);
 static void rb_playlist_source_set_property (GObject *object,
-			                  guint prop_id,
-			                  const GValue *value,
-			                  GParamSpec *pspec);
+					     guint prop_id,
+					     const GValue *value,
+					     GParamSpec *pspec);
 static void rb_playlist_source_get_property (GObject *object,
-			                  guint prop_id,
-			                  GValue *value,
-			                  GParamSpec *pspec);
+					     guint prop_id,
+					     GValue *value,
+					     GParamSpec *pspec);
 
 /* source methods */
-static const char *impl_get_browser_key (RBSource *source);
+static char *impl_get_browser_key (RBSource *source);
 static RBEntryView *impl_get_entry_view (RBSource *source);
-static void impl_move_to_trash (RBSource *asource);
 static void impl_song_properties (RBSource *source);
 static gboolean impl_show_popup (RBSource *source);
 
-static void rb_playlist_source_songs_show_popup_cb (RBEntryView *view, RBPlaylistSource *playlist_view);
+static void rb_playlist_source_songs_show_popup_cb (RBEntryView *view,
+						    gboolean over_entry,
+						    RBPlaylistSource *playlist_view);
 static void rb_playlist_source_drop_cb (GtkWidget *widget,
 				     GdkDragContext *context,
 				     gint x,
@@ -81,12 +81,12 @@ static void rb_playlist_source_drop_cb (GtkWidget *widget,
 				     guint time,
 				     gpointer user_data);
 
-
 static void rb_playlist_source_row_deleted (GtkTreeModel *model,
 					    GtkTreePath *path,
 					    RBPlaylistSource *playlist);
 static void default_show_entry_view_popup (RBPlaylistSource *source,
-					   RBEntryView *view);
+					   RBEntryView *view,
+					   gboolean over_entry);
 static void rb_playlist_source_entry_added_cb (RhythmDB *db, RhythmDBEntry *entry,
 					       RBPlaylistSource *source);
 static void rb_playlist_source_track_cell_data_func (GtkTreeViewColumn *column, GtkCellRenderer *renderer,
@@ -102,15 +102,13 @@ struct RBPlaylistSourcePrivate
 
 	GHashTable *entries;
 
-	RhythmDBEntryType entry_type;
 	RhythmDBQueryModel *model;
 
-	GtkWidget *vbox;
-
 	RBEntryView *songs;
 
 	gboolean dirty;
 	gboolean is_local;
+	gboolean dispose_has_run;
 
 	char *title;
 };
@@ -123,14 +121,12 @@ enum
 	PROP_DB,
 	PROP_DIRTY,
 	PROP_LOCAL,
-	PROP_ENTRY_TYPE
 };
 
 static const GtkTargetEntry target_uri [] = { { "text/uri-list", 0, 0 } };
 
 G_DEFINE_ABSTRACT_TYPE (RBPlaylistSource, rb_playlist_source, RB_TYPE_SOURCE);
 
-
 static void
 rb_playlist_source_class_init (RBPlaylistSourceClass *klass)
 {
@@ -153,10 +149,8 @@ rb_playlist_source_class_init (RBPlaylistSourceClass *klass)
 	source_class->impl_can_delete = (RBSourceFeatureFunc) rb_false_function;
 	source_class->impl_can_add_to_queue = (RBSourceFeatureFunc) rb_true_function;
 	source_class->impl_can_move_to_trash = (RBSourceFeatureFunc) rb_true_function;
-	source_class->impl_move_to_trash = impl_move_to_trash;
 	source_class->impl_song_properties = impl_song_properties;
 	source_class->impl_can_pause = (RBSourceFeatureFunc) rb_true_function;
-	source_class->impl_have_url = (RBSourceFeatureFunc) rb_false_function;
 	source_class->impl_show_popup = impl_show_popup;
 
 	klass->impl_show_entry_view_popup = default_show_entry_view_popup;
@@ -182,13 +176,6 @@ rb_playlist_source_class_init (RBPlaylistSourceClass *klass)
 							       "whether this playlist is attached to the local library",
 							       TRUE,
 							       G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
-	g_object_class_install_property (object_class,
-					 PROP_ENTRY_TYPE,
-					 g_param_spec_int ("entry-type",
-							   "entry-type",
-							   "The entry type this playlist accepts",
-							   -1, G_MAXINT, -1,
-							   G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 
 	g_type_class_add_private (klass, sizeof (RBPlaylistSourcePrivate));
 }
@@ -199,49 +186,77 @@ rb_playlist_source_init (RBPlaylistSource *source)
 	source->priv = RB_PLAYLIST_SOURCE_GET_PRIVATE (source);
 }
 
+static void
+rb_playlist_source_set_db (RBPlaylistSource *source,
+			   RhythmDB         *db)
+{
+	if (source->priv->db != NULL) {
+		g_signal_handlers_disconnect_by_func (source->priv->db,
+						      rb_playlist_source_entry_added_cb,
+						      source);
+		g_object_unref (source->priv->db);
+
+	}
+
+	source->priv->db = db;
+
+	if (source->priv->db != NULL) {
+		g_object_ref (source->priv->db);
+		g_signal_connect_object (G_OBJECT (source->priv->db), "entry_added",
+					 G_CALLBACK (rb_playlist_source_entry_added_cb),
+					 source, 0);
+	}
+
+}
+
 static GObject *
-rb_playlist_source_constructor (GType type, guint n_construct_properties,
+rb_playlist_source_constructor (GType type,
+				guint n_construct_properties,
 				GObjectConstructParam *construct_properties)
 {
 	GObject *shell_player;
 	RBPlaylistSource *source;
 	RBPlaylistSourceClass *klass;
 	RBShell *shell;
+	RhythmDB *db;
+	RhythmDBQueryModel *query_model;
 
 	klass = RB_PLAYLIST_SOURCE_CLASS (g_type_class_peek (RB_TYPE_PLAYLIST_SOURCE));
 
 	source = RB_PLAYLIST_SOURCE (G_OBJECT_CLASS (rb_playlist_source_parent_class)->
 			constructor (type, n_construct_properties, construct_properties));
 
-	g_object_get (G_OBJECT (source), "shell", &shell, NULL);
-	g_object_get (G_OBJECT (shell), "db", &source->priv->db, NULL);
+	g_object_get (source, "shell", &shell, NULL);
+	g_object_get (shell, "db", &db, NULL);
 	shell_player = rb_shell_get_player (shell);
-	g_object_unref (G_OBJECT (shell));
+	rb_playlist_source_set_db (source, db);
+	g_object_unref (db);
+	g_object_unref (shell);
 
-	source->priv->vbox = gtk_vbox_new (FALSE, 5);
+	source->priv->entries = g_hash_table_new_full (rb_refstring_hash, rb_refstring_equal,
+						       (GDestroyNotify)rb_refstring_unref, NULL);
 
-	gtk_container_add (GTK_CONTAINER (source), source->priv->vbox);
-
-	source->priv->entries = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
-
-	g_signal_connect_object (G_OBJECT (source->priv->db), "entry_added",
-				 G_CALLBACK (rb_playlist_source_entry_added_cb),
-				 source, 0);
-	g_signal_connect_object (G_OBJECT (source->priv->db), "entry_restored",
-				 G_CALLBACK (rb_playlist_source_entry_added_cb),
-				 source, 0);
-
-	source->priv->songs = rb_entry_view_new (source->priv->db, shell_player, 
+	source->priv->songs = rb_entry_view_new (source->priv->db,
+						 shell_player,
 					 	 NULL, TRUE, TRUE);
 
-	rb_playlist_source_set_query_model (source, rhythmdb_query_model_new_empty (source->priv->db));
+	query_model = rhythmdb_query_model_new_empty (source->priv->db);
+	rb_playlist_source_set_query_model (source, query_model);
+	g_object_unref (query_model);
 
 	{
-		const char *title = _("Trac_k");
+		const char *title = "";
 		const char *strings[3] = {0};
 
 		GtkTreeViewColumn *column = gtk_tree_view_column_new ();
 		GtkCellRenderer *renderer = gtk_cell_renderer_text_new ();
+
+		g_object_set(renderer,
+			     "style", PANGO_STYLE_OBLIQUE,
+			     "weight", PANGO_WEIGHT_LIGHT,
+			     "xalign", 1.0,
+			     NULL);
+
 		gtk_tree_view_column_pack_start (column, renderer, TRUE);
 
 		gtk_tree_view_column_set_resizable (column, TRUE);
@@ -255,26 +270,28 @@ rb_playlist_source_constructor (GType type, guint n_construct_properties,
 							 (GtkTreeCellDataFunc)
 							 rb_playlist_source_track_cell_data_func,
 							 source, NULL);
-		rb_entry_view_append_column_custom (source->priv->songs, column, title,
-						    "PlaylistTrack", NULL, 0);
+		rb_entry_view_insert_column_custom (source->priv->songs, column, title,
+						    "PlaylistTrack", NULL, 0, 0);
 	}
 
-	rb_entry_view_append_column (source->priv->songs, RB_ENTRY_VIEW_COL_TITLE);
-	rb_entry_view_append_column (source->priv->songs, RB_ENTRY_VIEW_COL_GENRE);
-	rb_entry_view_append_column (source->priv->songs, RB_ENTRY_VIEW_COL_ARTIST);
-	rb_entry_view_append_column (source->priv->songs, RB_ENTRY_VIEW_COL_ALBUM);
-	rb_entry_view_append_column (source->priv->songs, RB_ENTRY_VIEW_COL_YEAR);
-	rb_entry_view_append_column (source->priv->songs, RB_ENTRY_VIEW_COL_DURATION);
-	rb_entry_view_append_column (source->priv->songs, RB_ENTRY_VIEW_COL_RATING);
-	rb_entry_view_append_column (source->priv->songs, RB_ENTRY_VIEW_COL_PLAY_COUNT);
-	rb_entry_view_append_column (source->priv->songs, RB_ENTRY_VIEW_COL_LAST_PLAYED);
-	rb_entry_view_append_column (source->priv->songs, RB_ENTRY_VIEW_COL_FIRST_SEEN);
+	rb_entry_view_append_column (source->priv->songs, RB_ENTRY_VIEW_COL_TRACK_NUMBER, FALSE);
+	rb_entry_view_append_column (source->priv->songs, RB_ENTRY_VIEW_COL_TITLE, TRUE);
+	rb_entry_view_append_column (source->priv->songs, RB_ENTRY_VIEW_COL_GENRE, FALSE);
+	rb_entry_view_append_column (source->priv->songs, RB_ENTRY_VIEW_COL_ARTIST, FALSE);
+	rb_entry_view_append_column (source->priv->songs, RB_ENTRY_VIEW_COL_ALBUM, FALSE);
+	rb_entry_view_append_column (source->priv->songs, RB_ENTRY_VIEW_COL_YEAR, FALSE);
+	rb_entry_view_append_column (source->priv->songs, RB_ENTRY_VIEW_COL_DURATION, FALSE);
+ 	rb_entry_view_append_column (source->priv->songs, RB_ENTRY_VIEW_COL_QUALITY, FALSE);
+	rb_entry_view_append_column (source->priv->songs, RB_ENTRY_VIEW_COL_RATING, FALSE);
+	rb_entry_view_append_column (source->priv->songs, RB_ENTRY_VIEW_COL_PLAY_COUNT, FALSE);
+	rb_entry_view_append_column (source->priv->songs, RB_ENTRY_VIEW_COL_LAST_PLAYED, FALSE);
+	rb_entry_view_append_column (source->priv->songs, RB_ENTRY_VIEW_COL_FIRST_SEEN, FALSE);
 	rb_entry_view_set_columns_clickable (source->priv->songs, FALSE);
 
 	rb_playlist_source_setup_entry_view (source, source->priv->songs);
 
-	gtk_box_pack_start_defaults (GTK_BOX (source->priv->vbox), GTK_WIDGET (source->priv->songs));
-		
+	gtk_container_add (GTK_CONTAINER (source), GTK_WIDGET (source->priv->songs));
+
 	gtk_widget_show_all (GTK_WIDGET (source));
 
 	return G_OBJECT (source);
@@ -284,11 +301,27 @@ static void
 rb_playlist_source_dispose (GObject *object)
 {
 	RBPlaylistSource *source = RB_PLAYLIST_SOURCE (object);
-	if (source->priv->db) {
+
+	if (source->priv->dispose_has_run) {
+		/* If dispose did already run, return. */
+		rb_debug ("Dispose has already run for playlist source %p", object);
+		return;
+	}
+	/* Make sure dispose does not run twice. */
+	source->priv->dispose_has_run = TRUE;
+
+	rb_debug ("Disposing playlist source %p", source);
+
+	if (source->priv->db != NULL) {
 		g_object_unref (source->priv->db);
 		source->priv->db = NULL;
 	}
 
+	if (source->priv->model != NULL) {
+		g_object_unref (source->priv->model);
+		source->priv->model = NULL;
+	}
+
 	G_OBJECT_CLASS (rb_playlist_source_parent_class)->dispose (object);
 }
 
@@ -303,6 +336,8 @@ rb_playlist_source_finalize (GObject *object)
 	source = RB_PLAYLIST_SOURCE (object);
 	g_return_if_fail (source->priv != NULL);
 
+	rb_debug ("Finalizing playlist source %p", source);
+
 	g_hash_table_destroy (source->priv->entries);
 
 	g_free (source->priv->title);
@@ -313,20 +348,16 @@ rb_playlist_source_finalize (GObject *object)
 
 static void
 rb_playlist_source_set_property (GObject *object,
-			      guint prop_id,
-			      const GValue *value,
-			      GParamSpec *pspec)
+				 guint prop_id,
+				 const GValue *value,
+				 GParamSpec *pspec)
 {
 	RBPlaylistSource *source = RB_PLAYLIST_SOURCE (object);
 
-	switch (prop_id)
-	{
+	switch (prop_id) {
 	case PROP_LOCAL:
 		source->priv->is_local = g_value_get_boolean (value);
 		break;
-	case PROP_ENTRY_TYPE:
-		source->priv->entry_type = g_value_get_int (value);
-		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -335,14 +366,13 @@ rb_playlist_source_set_property (GObject *object,
 
 static void
 rb_playlist_source_get_property (GObject *object,
-			      guint prop_id,
-			      GValue *value,
-			      GParamSpec *pspec)
+				 guint prop_id,
+				 GValue *value,
+				 GParamSpec *pspec)
 {
 	RBPlaylistSource *source = RB_PLAYLIST_SOURCE (object);
 
-	switch (prop_id)
-	{
+	switch (prop_id) {
 	case PROP_DB:
 		g_value_set_object (value, source->priv->db);
 		break;
@@ -352,9 +382,6 @@ rb_playlist_source_get_property (GObject *object,
 	case PROP_LOCAL:
 		g_value_set_boolean (value, source->priv->is_local);
 		break;
-	case PROP_ENTRY_TYPE:
-		g_value_set_int (value, source->priv->entry_type);
-		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -362,22 +389,25 @@ rb_playlist_source_get_property (GObject *object,
 }
 
 static void
-default_show_entry_view_popup (RBPlaylistSource *source, RBEntryView *view)
+default_show_entry_view_popup (RBPlaylistSource *source,
+			       RBEntryView *view,
+			       gboolean over_entry)
 {
-	_rb_source_show_popup (RB_SOURCE (source), PLAYLIST_SOURCE_SONGS_POPUP_PATH);
+	if (over_entry)
+		_rb_source_show_popup (RB_SOURCE (source), PLAYLIST_SOURCE_SONGS_POPUP_PATH);
 }
 
 static void
 rb_playlist_source_songs_show_popup_cb (RBEntryView *view,
+					gboolean over_entry,
 					RBPlaylistSource *source)
 {
 	RBPlaylistSourceClass *klass = RB_PLAYLIST_SOURCE_GET_CLASS (source);
 	if (klass->impl_show_entry_view_popup)
-		klass->impl_show_entry_view_popup (source, view);
+		klass->impl_show_entry_view_popup (source, view, over_entry);
 }
 
-
-static const char *
+static char *
 impl_get_browser_key (RBSource *source)
 {
 	return NULL;
@@ -391,21 +421,6 @@ impl_get_entry_view (RBSource *asource)
 	return source->priv->songs;
 }
 
-static void
-impl_move_to_trash (RBSource *asource)
-{
-	RBPlaylistSource *source = RB_PLAYLIST_SOURCE (asource);
-	GList *sel, *tem;
-
-	sel = rb_entry_view_get_selected_entries (source->priv->songs);
-	for (tem = sel; tem != NULL; tem = tem->next) {
-		rhythmdb_entry_move_to_trash (source->priv->db,
-				(RhythmDBEntry *) tem->data);
-		rhythmdb_commit (source->priv->db);
-	}
-	g_list_free (sel);
-}
-
 static void
 impl_song_properties (RBSource *asource)
 {
@@ -466,8 +481,16 @@ playlist_iter_func (GtkTreeModel *model,
 
 	gtk_tree_model_get (model, iter, 0, &entry, -1);
 
-	*uri = g_strdup (entry->location);
-	*title = g_strdup (rb_refstring_get (entry->title));
+	if (uri != NULL) {
+		*uri = rhythmdb_entry_dup_string (entry, RHYTHMDB_PROP_LOCATION);
+	}
+	if (title != NULL) {
+		*title = rhythmdb_entry_dup_string (entry, RHYTHMDB_PROP_TITLE);
+	}
+
+	if (entry != NULL) {
+		rhythmdb_entry_unref (entry);
+	}
 }
 #else
 static void
@@ -482,97 +505,48 @@ playlist_iter_func (GtkTreeModel *model,
 
 	gtk_tree_model_get (model, iter, 0, &entry, -1);
 
-	*uri = g_strdup (entry->location);
-	*title = g_strdup (rb_refstring_get (entry->title));
-	*custom_title = FALSE;
+	if (uri != NULL) {
+		*uri = rhythmdb_entry_dup_string (entry, RHYTHMDB_PROP_LOCATION);
+	}
+	if (title != NULL) {
+		*title = rhythmdb_entry_dup_string (entry, RHYTHMDB_PROP_TITLE);
+	}
+	if (custom_title != NULL) {
+		*custom_title = FALSE;
+	}
+
+	if (entry != NULL) {
+		rhythmdb_entry_unref (entry);
+	}
 }
 #endif /* TOTEM_PL_PARSER_CHECK_VERSION */
 
 void
-rb_playlist_source_save_playlist (RBPlaylistSource *source, const char *uri)
+rb_playlist_source_save_playlist (RBPlaylistSource *source,
+				  const char *uri,
+				  gboolean m3u_format)
 {
 	TotemPlParser *playlist;
 	GError *error = NULL;
 	char *name;
 
+	g_return_if_fail (RB_IS_PLAYLIST_SOURCE (source));
+
 	rb_debug ("saving playlist");
 	playlist = totem_pl_parser_new ();
 
-	g_object_get (G_OBJECT (source), "name", &name, NULL);
+	g_object_get (source, "name", &name, NULL);
 
 	totem_pl_parser_write_with_title (playlist, GTK_TREE_MODEL (source->priv->model),
 					  playlist_iter_func, uri, name,
-					  TOTEM_PL_PARSER_PLS, NULL, &error);
+					  m3u_format ? TOTEM_PL_PARSER_M3U : TOTEM_PL_PARSER_PLS,
+					  NULL, &error);
 	g_free (name);
 	if (error != NULL)
 		rb_error_dialog (NULL, _("Couldn't save playlist"),
 				 "%s", error->message);
 }
 
-static gboolean
-burn_playlist_iter_func (GtkTreeModel *model, GtkTreeIter *iter, char **uri, char **artist, char **title, gulong *duration)
-{
-	RhythmDBEntry *entry;
-
-	gtk_tree_model_get (model, iter, 0, &entry, -1);
-
-	*uri = g_strdup (entry->location);
-	*artist = g_strdup (rb_refstring_get (entry->artist));
-	*title = g_strdup (rb_refstring_get (entry->title));
-	*duration = entry->duration;
-
-	return TRUE;
-}
-
-void
-rb_playlist_source_burn_playlist (RBPlaylistSource *source)
-{
-	GtkWidget *recorder;
-	GtkWidget *parent;
-	char *name;
-	RBShell *shell;
-	gboolean res;
-	GError *error;
-
-	/* don't burn if the playlist is empty */
-	if (gtk_tree_model_iter_n_children (GTK_TREE_MODEL (source->priv->model), NULL) == 0)
-		return;
-
-	rb_debug ("burning playlist");
-
-	g_object_get (source, "name", &name, "shell", &shell, NULL);
-
-	parent = gtk_widget_get_toplevel (GTK_WIDGET (source));
-	recorder = rb_playlist_source_recorder_new (parent,
-						    shell,
-						    name);
-	g_object_unref (shell);
-	g_free (name);
-
-	error = NULL;
-	res = rb_playlist_source_recorder_add_from_model (RB_PLAYLIST_SOURCE_RECORDER (recorder),
-							  GTK_TREE_MODEL (source->priv->model),
-							  burn_playlist_iter_func,
-							  &error);
-	if (! res) {
-		rb_error_dialog (GTK_WINDOW (parent),
-				 _("Unable to create audio CD"),
-				 "%s", error->message);
-		g_error_free (error);
-
-		gtk_widget_destroy (recorder);
-
-		return;
-	}
-
-        g_signal_connect (recorder,
-			  "response",
-			  G_CALLBACK (gtk_widget_destroy),
-			  NULL);
-
-	gtk_widget_show (recorder);
-}
-
 /* Adapted from yelp-toc-pager.c */
 static xmlChar *
 xml_get_and_trim_names (xmlNodePtr node)
@@ -658,14 +632,17 @@ RBSource *
 rb_playlist_source_new_from_xml	(RBShell *shell,
 				 xmlNodePtr node)
 {
+	RBSource *source = NULL;
 	xmlChar *tmp;
 	xmlChar *name;
-	RBSource *source;
+
+	g_return_val_if_fail (RB_IS_SHELL (shell), NULL);
 
 	/* Try to get name from XML and remove translated names */
 	name = get_playlist_name_from_xml (node);
 
 	tmp = xmlGetProp (node, RB_PLAYLIST_TYPE);
+
 	if (!xmlStrcmp (tmp, RB_PLAYLIST_AUTOMATIC))
 		source = rb_auto_playlist_source_new_from_xml (shell, node);
 	else if (!xmlStrcmp (tmp, RB_PLAYLIST_STATIC))
@@ -675,26 +652,33 @@ rb_playlist_source_new_from_xml	(RBShell *shell,
 
 		g_object_get (shell, "queue-source", &queue, NULL);
 		rb_static_playlist_source_load_from_xml (queue, node);
-		g_object_unref (G_OBJECT (queue));
-		return NULL;
-	} else
-		g_assert_not_reached ();
+		g_object_unref (queue);
+	} else {
+		g_warning ("attempting to load playlist '%s' of unknown type '%s'", name, tmp);
+	}
 
-	g_object_set (G_OBJECT (source), "name", name, NULL);
-	g_free (name);
+	if (source != NULL) {
+		g_object_set (G_OBJECT (source), "name", name, NULL);
+	}
+
+	xmlFree (name);
+	xmlFree (tmp);
 
 	return source;
 }
 
 void
-rb_playlist_source_save_to_xml (RBPlaylistSource *source, xmlNodePtr parent_node)
+rb_playlist_source_save_to_xml (RBPlaylistSource *source,
+				xmlNodePtr parent_node)
 {
 	xmlNodePtr node;
 	xmlChar *name;
 	RBPlaylistSourceClass *klass = RB_PLAYLIST_SOURCE_GET_CLASS (source);
 
+	g_return_if_fail (RB_IS_PLAYLIST_SOURCE (source));
+
 	node = xmlNewChild (parent_node, NULL, RB_PLAYLIST_PLAYLIST, NULL);
-	g_object_get (G_OBJECT (source), "name", &name, NULL);
+	g_object_get (source, "name", &name, NULL);
 	xmlSetProp (node, RB_PLAYLIST_NAME, name);
 	g_free (name);
 
@@ -703,35 +687,47 @@ rb_playlist_source_save_to_xml (RBPlaylistSource *source, xmlNodePtr parent_node
 	source->priv->dirty = FALSE;
 }
 
-
 static void
 rb_playlist_source_row_deleted (GtkTreeModel *model,
 				GtkTreePath *path,
 				RBPlaylistSource *source)
 {
-	RhythmDBEntry *entry = 
-		rhythmdb_query_model_tree_path_to_entry (RHYTHMDB_QUERY_MODEL (model),
+	RhythmDBEntry *entry;
+	RBRefString *location;
+
+	entry = rhythmdb_query_model_tree_path_to_entry (RHYTHMDB_QUERY_MODEL (model),
 							 path);
-	g_hash_table_remove (source->priv->entries, entry->location);
+
+	location = rhythmdb_entry_get_refstring (entry, RHYTHMDB_PROP_LOCATION);
+	if (g_hash_table_remove (source->priv->entries, location))
+		source->priv->dirty = TRUE;
+
+	rb_refstring_unref (location);
+	rhythmdb_entry_unref (entry);
 }
 
 static void
-rb_playlist_source_entry_added_cb (RhythmDB *db, 
+rb_playlist_source_entry_added_cb (RhythmDB *db,
 				   RhythmDBEntry *entry,
 				   RBPlaylistSource *source)
 {
-	const char *location;
+	RBRefString *location;
+
+	location = rhythmdb_entry_get_refstring (entry, RHYTHMDB_PROP_LOCATION);
 
-	location = entry->location;
 	if (g_hash_table_lookup (source->priv->entries, location)) {
 		rhythmdb_query_model_add_entry (source->priv->model, entry, -1);
 		source->priv->dirty = TRUE;
 	}
+
+	rb_refstring_unref (location);
 }
 
 static void
-rb_playlist_source_track_cell_data_func (GtkTreeViewColumn *column, GtkCellRenderer *renderer,
-					 GtkTreeModel *tree_model, GtkTreeIter *iter,
+rb_playlist_source_track_cell_data_func (GtkTreeViewColumn *column,
+					 GtkCellRenderer *renderer,
+					 GtkTreeModel *tree_model,
+					 GtkTreeIter *iter,
 					 RBPlaylistSource *source)
 {
 	char *str;
@@ -752,56 +748,70 @@ void
 rb_playlist_source_setup_entry_view (RBPlaylistSource *source,
 				     RBEntryView *entry_view)
 {
-	g_signal_connect_object (G_OBJECT (entry_view), "show_popup",
+	g_return_if_fail (RB_IS_PLAYLIST_SOURCE (source));
+
+	g_signal_connect_object (entry_view, "show_popup",
 				 G_CALLBACK (rb_playlist_source_songs_show_popup_cb), source, 0);
-	g_signal_connect_object (G_OBJECT (entry_view), "drag_data_received",
+	g_signal_connect_object (entry_view, "drag_data_received",
 				 G_CALLBACK (rb_playlist_source_drop_cb), source, 0);
-	gtk_drag_dest_set (GTK_WIDGET (entry_view), GTK_DEST_DEFAULT_ALL,
-			   target_uri, G_N_ELEMENTS (target_uri), GDK_ACTION_COPY);
+	gtk_drag_dest_set (GTK_WIDGET (entry_view),
+			   GTK_DEST_DEFAULT_ALL,
+			   target_uri,
+			   G_N_ELEMENTS (target_uri),
+			   GDK_ACTION_COPY);
 }
 
 void
 rb_playlist_source_set_query_model (RBPlaylistSource *source,
 				    RhythmDBQueryModel *model)
 {
-	if (source->priv->model) {
+	g_return_if_fail (RB_IS_PLAYLIST_SOURCE (source));
+
+	if (source->priv->model != NULL) {
 		/* if the query model is replaced, the set of entries in
 		 * the playlist will change, so we should mark the playlist dirty.
 		 */
 		source->priv->dirty = TRUE;
-		g_signal_handlers_disconnect_by_func (G_OBJECT (source->priv->model),
+		g_signal_handlers_disconnect_by_func (source->priv->model,
 						      G_CALLBACK (rb_playlist_source_row_deleted),
 						      source);
 		g_object_unref (source->priv->model);
 	}
+
 	source->priv->model = model;
 
-	if (model) {
-		g_object_ref (G_OBJECT (model));
-		g_signal_connect_object (G_OBJECT (source->priv->model), "row_deleted",
+	if (source->priv->model != NULL) {
+		g_object_ref (source->priv->model);
+		g_signal_connect_object (source->priv->model, "row_deleted",
 					 G_CALLBACK (rb_playlist_source_row_deleted), source, 0);
 	}
 
 	rb_entry_view_set_model (source->priv->songs, RHYTHMDB_QUERY_MODEL (source->priv->model));
-	
-	g_object_set (G_OBJECT (source), "query-model", source->priv->model, NULL);
+
+	g_object_set (source, "query-model", source->priv->model, NULL);
 }
 
 RhythmDB *
 rb_playlist_source_get_db (RBPlaylistSource *source)
 {
+	g_return_val_if_fail (RB_IS_PLAYLIST_SOURCE (source), NULL);
+
 	return source->priv->db;
 }
 
 RhythmDBQueryModel *
 rb_playlist_source_get_query_model (RBPlaylistSource *source)
 {
+	g_return_val_if_fail (RB_IS_PLAYLIST_SOURCE (source), NULL);
+
 	return source->priv->model;
 }
 
 void
 rb_playlist_source_mark_dirty (RBPlaylistSource *source)
 {
+	g_return_if_fail (RB_IS_PLAYLIST_SOURCE (source));
+
 	source->priv->dirty = TRUE;
 }
 
@@ -809,19 +819,38 @@ gboolean
 rb_playlist_source_location_in_map (RBPlaylistSource *source,
 				    const char *location)
 {
-	return (g_hash_table_lookup (source->priv->entries, location) != NULL);
+	RBRefString *refstr;
+	gboolean found;
+
+	g_return_val_if_fail (RB_IS_PLAYLIST_SOURCE (source), FALSE);
+
+	refstr = rb_refstring_find (location);
+	if (refstr == NULL) {
+		return FALSE;
+	}
+
+	found = (g_hash_table_lookup (source->priv->entries, refstr) != NULL);
+	rb_refstring_unref (refstr);
+
+	return found;
 }
 
 gboolean
 rb_playlist_source_add_to_map (RBPlaylistSource *source,
 			       const char *location)
 {
-	if (g_hash_table_lookup (source->priv->entries, location)) {
+	RBRefString *refstr;
+
+	g_return_val_if_fail (RB_IS_PLAYLIST_SOURCE (source), FALSE);
+
+	refstr = rb_refstring_new (location);
+	if (g_hash_table_lookup (source->priv->entries, refstr)) {
+		rb_refstring_unref (refstr);
 		return FALSE;
 	}
 
 	g_hash_table_insert (source->priv->entries,
-			     g_strdup (location), GINT_TO_POINTER (1));
+			     refstr, GINT_TO_POINTER (1));
+
 	return TRUE;
 }
-
